from __future__ import annotations

import logging
import time
from functools import cached_property
from typing import Callable, Generator, List, Any

from vellum import TestSuiteRunRead, TestSuiteRunMetricOutput_Number
from vellum.client import Vellum
from vellum.lib.test_suites.constants import (
    DEFAULT_MAX_POLLING_DURATION_MS,
    DEFAULT_POLLING_INTERVAL_MS,
)
from vellum.lib.test_suites.exceptions import TestSuiteRunResultsException
from vellum.lib.utils.env import get_api_key
from vellum.lib.utils.paginator import PaginatedResults, get_all_results
from vellum.types import (
    ExternalTestCaseExecutionRequest,
    NamedTestCaseVariableValueRequest,
    TestCaseVariableValue,
    TestSuiteRunExecConfigRequest_External,
    TestSuiteRunExecution,
    TestSuiteRunExternalExecConfigDataRequest,
    TestSuiteRunMetricOutput,
    TestSuiteRunState,
)

logger = logging.getLogger(__name__)


class VellumTestSuiteRunExecution(TestSuiteRunExecution):
    @classmethod
    def from_api(cls, execution: TestSuiteRunExecution) -> VellumTestSuiteRunExecution:
        return cls(
            id=execution.id,
            test_case_id=execution.test_case_id,
            outputs=execution.outputs,
            metric_results=execution.metric_results,
        )

    def get_metric_output(
        self,
        metric_identifier: str | None = None,
        output_identifier: str | None = None,
    ) -> TestSuiteRunMetricOutput:
        """Retrieve a metric's output by specifying the info needed to uniquely identify that metric and output.

        metric_identifier: Anything that uniquely identifies the metric to retrieve (e.g. it's label, name or id)
        output_identifier: Anything that uniquely identifies the output to retrieve (e.g. it's name)
        """

        metric_outputs = self.get_metric_outputs(metric_identifier)

        filtered_metric_outputs = [
            metric_output
            for metric_output in metric_outputs
            if output_identifier is None or metric_output.name == output_identifier
        ]

        if len(filtered_metric_outputs) == 0:
            raise TestSuiteRunResultsException(
                f"No metric outputs found with identifier: {output_identifier}"
            )

        if len(filtered_metric_outputs) > 1:
            raise TestSuiteRunResultsException(
                f"Multiple metric outputs found with identifier: {output_identifier}"
            )

        return filtered_metric_outputs[0]

    def get_metric_outputs(
        self,
        metric_identifier: str | None = None,
    ) -> List[TestSuiteRunMetricOutput]:
        """Return a metric's output across all executions by providing the info needed to uniquely identify it.

        metric_identifier: Anything that uniquely identifies the metric to retrieve (e.g. it's label, name or id).
        """

        filtered_metric_results = [
            metric_output
            for metric_output in self.metric_results
            if metric_identifier is None
            or (metric_output.metric_id == metric_identifier)
            or (metric_output.metric_label == metric_identifier)
            or (
                metric_output.metric_definition
                and metric_output.metric_definition.id == metric_identifier
            )
            or (
                metric_output.metric_definition
                and metric_output.metric_definition.name == metric_identifier
            )
            or (
                metric_output.metric_definition
                and metric_output.metric_definition.label == metric_identifier
            )
        ]

        if len(filtered_metric_results) == 0:
            raise TestSuiteRunResultsException(
                f"No metric results found with identifier: {metric_identifier}"
            )

        if len(filtered_metric_results) > 1:
            raise TestSuiteRunResultsException(
                f"Multiple metric results found with identifier: {metric_identifier}"
            )

        return filtered_metric_results[0].outputs


class VellumTestSuiteRunResults:
    """A utility class with methods for conveniently operating on a Test Suite Run and its results."""

    def __init__(
        self,
        test_suite_run: TestSuiteRunRead,
        *,
        client: Vellum | None = None,
        polling_interval: int = DEFAULT_POLLING_INTERVAL_MS,
        max_polling_duration: int = DEFAULT_MAX_POLLING_DURATION_MS,
    ) -> None:
        self._test_suite_run = test_suite_run
        self._client = client or Vellum(
            api_key=get_api_key(),
        )
        self._executions: Generator[VellumTestSuiteRunExecution, None, None] | None = (
            None
        )
        self._polling_interval = polling_interval
        self._max_polling_duration = max_polling_duration

    @property
    def state(self) -> TestSuiteRunState:
        return self._test_suite_run.state

    @cached_property
    def all_executions(self) -> list[VellumTestSuiteRunExecution]:
        return list(self._get_test_suite_run_executions())

    def get_metric_outputs(
        self, metric_identifier: str | None = None, output_identifier: str | None = None
    ) -> List[TestSuiteRunMetricOutput]:
        """Retrieve a metric's output across all executions by providing the info needed to uniquely identify it."""

        return [
            execution.get_metric_output(
                metric_identifier=metric_identifier, output_identifier=output_identifier
            )
            for execution in self.all_executions
        ]

    def get_count_metric_outputs(
        self,
        metric_identifier: str | None = None,
        output_identifier: str | None = None,
        *,
        predicate: Callable[[TestSuiteRunMetricOutput], bool] | None = None,
    ) -> int:
        """Returns the count of all metric outputs that match the given criteria."""

        metric_outputs = self.get_metric_outputs(
            metric_identifier=metric_identifier, output_identifier=output_identifier
        )

        if predicate is None:
            return len(metric_outputs)

        return len([output for output in metric_outputs if predicate(output)])

    def get_numeric_metric_output_values(
        self,
        metric_identifier: str | None = None,
        output_identifier: str | None = None,
    ) -> List[float]:
        """Returns the values of a numeric metric output that match the given criteria."""

        metric_outputs: list[TestSuiteRunMetricOutput_Number] = []

        for output in self.get_metric_outputs(
            metric_identifier=metric_identifier, output_identifier=output_identifier
        ):
            if output.type != "NUMBER":
                raise TestSuiteRunResultsException(
                    f"Expected a numeric metric output, but got a {output.type} output instead."
                )

            metric_outputs.append(output)

        return [output.value for output in metric_outputs]

    def get_mean_metric_output(
        self, metric_identifier: str | None = None, output_identifier: str | None = None
    ) -> float:
        """Returns the mean of all metric outputs that match the given criteria."""
        output_values = self.get_numeric_metric_output_values(
            metric_identifier=metric_identifier, output_identifier=output_identifier
        )
        return sum(output_values) / len(output_values)

    def get_min_metric_output(
        self, metric_identifier: str | None = None, output_identifier: str | None = None
    ) -> float:
        """Returns the min value across= all metric outputs that match the given criteria."""
        output_values = self.get_numeric_metric_output_values(
            metric_identifier=metric_identifier, output_identifier=output_identifier
        )
        return min(output_values)

    def get_max_metric_output(
        self, metric_identifier: str | None = None, output_identifier: str | None = None
    ) -> float:
        """Returns the max value across all metric outputs that match the given criteria."""
        output_values = self.get_numeric_metric_output_values(
            metric_identifier=metric_identifier, output_identifier=output_identifier
        )
        return max(output_values)

    def wait_until_complete(self) -> None:
        """Wait until the Test Suite Run is no longer in a QUEUED or RUNNING state."""

        start_time = time.time_ns()
        while True:
            logger.debug("Polling for latest test suite run state...")
            self._refresh_test_suite_run()
            if self.state not in {"QUEUED", "RUNNING"}:
                break

            current_time = time.time_ns()
            if ((current_time - start_time) / 1e6) > self._max_polling_duration:
                raise TestSuiteRunResultsException(
                    "Test suite run timed out polling for executions"
                )

            time.sleep(self._polling_interval / 1000.0)

        if self.state == "FAILED":
            raise TestSuiteRunResultsException("Test suite run failed")

        if self.state == "CANCELLED":
            raise TestSuiteRunResultsException("Test suite run was cancelled")

    def _refresh_test_suite_run(self):
        test_suite_run = self._client.test_suite_runs.retrieve(self._test_suite_run.id)
        self._test_suite_run = test_suite_run

    def _list_paginated_executions(
        self, offset: int | None, limit: int | None
    ) -> PaginatedResults[TestSuiteRunExecution]:
        response = self._client.test_suite_runs.list_executions(
            self._test_suite_run.id,
            offset=offset,
            limit=limit,
            expand=[
                "results.metric_results.metric_definition",
                "results.metric_results.metric_label",
            ],
        )
        return PaginatedResults(results=response.results, count=response.count)

    def _wrap_api_executions(
        self, executions: Generator[TestSuiteRunExecution, None, None]
    ) -> Generator[VellumTestSuiteRunExecution, None, None]:
        for execution in executions:
            yield VellumTestSuiteRunExecution.from_api(execution)

    def _get_test_suite_run_executions(
        self,
    ) -> Generator[VellumTestSuiteRunExecution, None, None]:
        if self._executions is not None:
            return self._executions

        self.wait_until_complete()

        raw_api_executions = get_all_results(self._list_paginated_executions)
        self._executions = self._wrap_api_executions(raw_api_executions)
        return self._executions


class VellumTestSuite:
    """A utility class that provides methods for running a Vellum Test Suite and interacting with its results."""

    def __init__(
        self,
        test_suite_id: str,
        *,
        client: Vellum | None = None,
    ) -> None:
        self.client = client or Vellum(
            api_key=get_api_key(),
        )
        self._test_suite_id = test_suite_id

    def run_external(
        self,
        executable: Callable[
            [List[TestCaseVariableValue]], List[NamedTestCaseVariableValueRequest]
        ],
    ) -> VellumTestSuiteRunResults:
        """
        Runs this Vellum Test Suite on any executable function defined external to Vellum.

        Returns a wrapper that polls the generated Test Suite Run until it's done running and returns its results.
        """
        test_cases = self.client.test_suites.list_test_suite_test_cases(
            id=self._test_suite_id
        )
        executions: List[ExternalTestCaseExecutionRequest] = []

        for test_case in test_cases.results:
            outputs = executable(test_case.input_values)

            executions.append(
                ExternalTestCaseExecutionRequest(
                    test_case_id=test_case.id,  # type: ignore[arg-type]
                    outputs=outputs,
                )
            )

        test_suite_run = self.client.test_suite_runs.create(
            test_suite_id=self._test_suite_id,
            exec_config=TestSuiteRunExecConfigRequest_External(
                type="EXTERNAL",
                data=TestSuiteRunExternalExecConfigDataRequest(
                    executions=executions,
                ),
            ),
        )
        return VellumTestSuiteRunResults(test_suite_run, client=self.client)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .error_variable_value import ErrorVariableValue
from .function_call_variable_value import FunctionCallVariableValue
from .image_variable_value import ImageVariableValue
from .json_variable_value import JsonVariableValue
from .number_variable_value import NumberVariableValue
from .string_variable_value import StringVariableValue


class ArrayVariableValueItem_String(StringVariableValue):
    type: typing.Literal["STRING"] = "STRING"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class ArrayVariableValueItem_Number(NumberVariableValue):
    type: typing.Literal["NUMBER"] = "NUMBER"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class ArrayVariableValueItem_Json(JsonVariableValue):
    type: typing.Literal["JSON"] = "JSON"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class ArrayVariableValueItem_Error(ErrorVariableValue):
    type: typing.Literal["ERROR"] = "ERROR"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class ArrayVariableValueItem_FunctionCall(FunctionCallVariableValue):
    type: typing.Literal["FUNCTION_CALL"] = "FUNCTION_CALL"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class ArrayVariableValueItem_Image(ImageVariableValue):
    type: typing.Literal["IMAGE"] = "IMAGE"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


ArrayVariableValueItem = typing.Union[
    ArrayVariableValueItem_String,
    ArrayVariableValueItem_Number,
    ArrayVariableValueItem_Json,
    ArrayVariableValueItem_Error,
    ArrayVariableValueItem_FunctionCall,
    ArrayVariableValueItem_Image,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .basic_vectorizer_intfloat_multilingual_e_5_large import BasicVectorizerIntfloatMultilingualE5Large
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_cos_v_1 import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1,
)
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_dot_v_1 import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1,
)
from .hkunlp_instructor_xl_vectorizer import HkunlpInstructorXlVectorizer
from .open_ai_vectorizer_text_embedding_3_large import OpenAiVectorizerTextEmbedding3Large
from .open_ai_vectorizer_text_embedding_3_small import OpenAiVectorizerTextEmbedding3Small
from .open_ai_vectorizer_text_embedding_ada_002 import OpenAiVectorizerTextEmbeddingAda002


class IndexingConfigVectorizer_TextEmbedding3Small(OpenAiVectorizerTextEmbedding3Small):
    model_name: typing.Literal["text-embedding-3-small"] = "text-embedding-3-small"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_TextEmbedding3Large(OpenAiVectorizerTextEmbedding3Large):
    model_name: typing.Literal["text-embedding-3-large"] = "text-embedding-3-large"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_TextEmbeddingAda002(OpenAiVectorizerTextEmbeddingAda002):
    model_name: typing.Literal["text-embedding-ada-002"] = "text-embedding-ada-002"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_IntfloatMultilingualE5Large(BasicVectorizerIntfloatMultilingualE5Large):
    model_name: typing.Literal["intfloat/multilingual-e5-large"] = "intfloat/multilingual-e5-large"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseCosV1(
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1
):
    model_name: typing.Literal[
        "sentence-transformers/multi-qa-mpnet-base-cos-v1"
    ] = "sentence-transformers/multi-qa-mpnet-base-cos-v1"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseDotV1(
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1
):
    model_name: typing.Literal[
        "sentence-transformers/multi-qa-mpnet-base-dot-v1"
    ] = "sentence-transformers/multi-qa-mpnet-base-dot-v1"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class IndexingConfigVectorizer_HkunlpInstructorXl(HkunlpInstructorXlVectorizer):
    model_name: typing.Literal["hkunlp/instructor-xl"] = "hkunlp/instructor-xl"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


IndexingConfigVectorizer = typing.Union[
    IndexingConfigVectorizer_TextEmbedding3Small,
    IndexingConfigVectorizer_TextEmbedding3Large,
    IndexingConfigVectorizer_TextEmbeddingAda002,
    IndexingConfigVectorizer_IntfloatMultilingualE5Large,
    IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseCosV1,
    IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseDotV1,
    IndexingConfigVectorizer_HkunlpInstructorXl,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .reducto_chunking_request import ReductoChunkingRequest
from .sentence_chunking_request import SentenceChunkingRequest
from .token_overlapping_window_chunking_request import TokenOverlappingWindowChunkingRequest


class DocumentIndexChunkingRequest_ReductoChunker(ReductoChunkingRequest):
    chunker_name: typing.Literal["reducto-chunker"] = "reducto-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class DocumentIndexChunkingRequest_SentenceChunker(SentenceChunkingRequest):
    chunker_name: typing.Literal["sentence-chunker"] = "sentence-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class DocumentIndexChunkingRequest_TokenOverlappingWindowChunker(TokenOverlappingWindowChunkingRequest):
    chunker_name: typing.Literal["token-overlapping-window-chunker"] = "token-overlapping-window-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


DocumentIndexChunkingRequest = typing.Union[
    DocumentIndexChunkingRequest_ReductoChunker,
    DocumentIndexChunkingRequest_SentenceChunker,
    DocumentIndexChunkingRequest_TokenOverlappingWindowChunker,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .reducto_chunking import ReductoChunking
from .sentence_chunking import SentenceChunking
from .token_overlapping_window_chunking import TokenOverlappingWindowChunking


class DocumentIndexChunking_ReductoChunker(ReductoChunking):
    chunker_name: typing.Literal["reducto-chunker"] = "reducto-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class DocumentIndexChunking_SentenceChunker(SentenceChunking):
    chunker_name: typing.Literal["sentence-chunker"] = "sentence-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class DocumentIndexChunking_TokenOverlappingWindowChunker(TokenOverlappingWindowChunking):
    chunker_name: typing.Literal["token-overlapping-window-chunker"] = "token-overlapping-window-chunker"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


DocumentIndexChunking = typing.Union[
    DocumentIndexChunking_ReductoChunker,
    DocumentIndexChunking_SentenceChunker,
    DocumentIndexChunking_TokenOverlappingWindowChunker,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .named_test_case_chat_history_variable_value_request import NamedTestCaseChatHistoryVariableValueRequest
from .named_test_case_error_variable_value_request import NamedTestCaseErrorVariableValueRequest
from .named_test_case_function_call_variable_value_request import NamedTestCaseFunctionCallVariableValueRequest
from .named_test_case_json_variable_value_request import NamedTestCaseJsonVariableValueRequest
from .named_test_case_number_variable_value_request import NamedTestCaseNumberVariableValueRequest
from .named_test_case_search_results_variable_value_request import NamedTestCaseSearchResultsVariableValueRequest
from .named_test_case_string_variable_value_request import NamedTestCaseStringVariableValueRequest


class NamedTestCaseVariableValueRequest_String(NamedTestCaseStringVariableValueRequest):
    type: typing.Literal["STRING"] = "STRING"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_Number(NamedTestCaseNumberVariableValueRequest):
    type: typing.Literal["NUMBER"] = "NUMBER"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_Json(NamedTestCaseJsonVariableValueRequest):
    type: typing.Literal["JSON"] = "JSON"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_ChatHistory(NamedTestCaseChatHistoryVariableValueRequest):
    type: typing.Literal["CHAT_HISTORY"] = "CHAT_HISTORY"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_SearchResults(NamedTestCaseSearchResultsVariableValueRequest):
    type: typing.Literal["SEARCH_RESULTS"] = "SEARCH_RESULTS"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_Error(NamedTestCaseErrorVariableValueRequest):
    type: typing.Literal["ERROR"] = "ERROR"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class NamedTestCaseVariableValueRequest_FunctionCall(NamedTestCaseFunctionCallVariableValueRequest):
    type: typing.Literal["FUNCTION_CALL"] = "FUNCTION_CALL"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


NamedTestCaseVariableValueRequest = typing.Union[
    NamedTestCaseVariableValueRequest_String,
    NamedTestCaseVariableValueRequest_Number,
    NamedTestCaseVariableValueRequest_Json,
    NamedTestCaseVariableValueRequest_ChatHistory,
    NamedTestCaseVariableValueRequest_SearchResults,
    NamedTestCaseVariableValueRequest_Error,
    NamedTestCaseVariableValueRequest_FunctionCall,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import pydantic_v1
from .deployment_release_tag_deployment_history_item import DeploymentReleaseTagDeploymentHistoryItem
from .release_tag_source import ReleaseTagSource


class DeploymentReleaseTagRead(pydantic_v1.BaseModel):
    name: str = pydantic_v1.Field()
    """
    The name of the Release Tag
    """

    source: ReleaseTagSource = pydantic_v1.Field()
    """
    The source of how the Release Tag was originally created
    
    - `SYSTEM` - System
    - `USER` - User
    """

    history_item: DeploymentReleaseTagDeploymentHistoryItem = pydantic_v1.Field()
    """
    The Deployment History Item that this Release Tag is associated with
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import pydantic_v1
from .indexing_state_enum import IndexingStateEnum


class DocumentDocumentToDocumentIndex(pydantic_v1.BaseModel):
    id: str = pydantic_v1.Field()
    """
    Vellum-generated ID that uniquely identifies this link.
    """

    document_index_id: str = pydantic_v1.Field()
    """
    Vellum-generated ID that uniquely identifies the index this document is included in.
    """

    indexing_state: typing.Optional[IndexingStateEnum] = pydantic_v1.Field(default=None)
    """
    An enum value representing where this document is along its indexing lifecycle for this index.
    
    - `AWAITING_PROCESSING` - Awaiting Processing
    - `QUEUED` - Queued
    - `INDEXING` - Indexing
    - `INDEXED` - Indexed
    - `FAILED` - Failed
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

import abc
import builtins
import datetime
import enum
import typing

import jsii
import publication
import typing_extensions

from typeguard import check_type

from ._jsii import *

__all__ = [
    "data_vellum_document_index",
    "document_index",
    "provider",
]

publication.publish()

# Loading modules to ensure their types are registered with the jsii runtime library



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .workflow_execution_node_result_event import WorkflowExecutionNodeResultEvent
from .workflow_execution_workflow_result_event import WorkflowExecutionWorkflowResultEvent


class WorkflowStreamEvent_Workflow(WorkflowExecutionWorkflowResultEvent):
    type: typing.Literal["WORKFLOW"] = "WORKFLOW"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class WorkflowStreamEvent_Node(WorkflowExecutionNodeResultEvent):
    type: typing.Literal["NODE"] = "NODE"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


WorkflowStreamEvent = typing.Union[WorkflowStreamEvent_Workflow, WorkflowStreamEvent_Node]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .workflow_request_chat_history_input_request import WorkflowRequestChatHistoryInputRequest
from .workflow_request_json_input_request import WorkflowRequestJsonInputRequest
from .workflow_request_number_input_request import WorkflowRequestNumberInputRequest
from .workflow_request_string_input_request import WorkflowRequestStringInputRequest


class WorkflowRequestInputRequest_String(WorkflowRequestStringInputRequest):
    type: typing.Literal["STRING"] = "STRING"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class WorkflowRequestInputRequest_Json(WorkflowRequestJsonInputRequest):
    type: typing.Literal["JSON"] = "JSON"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class WorkflowRequestInputRequest_ChatHistory(WorkflowRequestChatHistoryInputRequest):
    type: typing.Literal["CHAT_HISTORY"] = "CHAT_HISTORY"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class WorkflowRequestInputRequest_Number(WorkflowRequestNumberInputRequest):
    type: typing.Literal["NUMBER"] = "NUMBER"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


WorkflowRequestInputRequest = typing.Union[
    WorkflowRequestInputRequest_String,
    WorkflowRequestInputRequest_Json,
    WorkflowRequestInputRequest_ChatHistory,
    WorkflowRequestInputRequest_Number,
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AddOpenaiApiKeyEnum,
    ApiNodeResult,
    ApiNodeResultData,
    ArrayChatMessageContent,
    ArrayChatMessageContentItem,
    ArrayChatMessageContentItemRequest,
    ArrayChatMessageContentItemRequest_FunctionCall,
    ArrayChatMessageContentItemRequest_Image,
    ArrayChatMessageContentItemRequest_String,
    ArrayChatMessageContentItem_FunctionCall,
    ArrayChatMessageContentItem_Image,
    ArrayChatMessageContentItem_String,
    ArrayChatMessageContentRequest,
    ArrayEnum,
    ArrayVariableValueItem,
    ArrayVariableValueItem_Error,
    ArrayVariableValueItem_FunctionCall,
    ArrayVariableValueItem_Image,
    ArrayVariableValueItem_Json,
    ArrayVariableValueItem_Number,
    ArrayVariableValueItem_String,
    ArrayVellumValueItem,
    ArrayVellumValueItem_Error,
    ArrayVellumValueItem_FunctionCall,
    ArrayVellumValueItem_Image,
    ArrayVellumValueItem_Json,
    ArrayVellumValueItem_Number,
    ArrayVellumValueItem_String,
    BasicVectorizerIntfloatMultilingualE5Large,
    BasicVectorizerIntfloatMultilingualE5LargeRequest,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1,
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request,
    ChatHistoryEnum,
    ChatHistoryInputRequest,
    ChatMessage,
    ChatMessageContent,
    ChatMessageContentRequest,
    ChatMessageContentRequest_Array,
    ChatMessageContentRequest_FunctionCall,
    ChatMessageContentRequest_Image,
    ChatMessageContentRequest_String,
    ChatMessageContent_Array,
    ChatMessageContent_FunctionCall,
    ChatMessageContent_Image,
    ChatMessageContent_String,
    ChatMessageRequest,
    ChatMessageRole,
    CodeExecutionNodeArrayResult,
    CodeExecutionNodeChatHistoryResult,
    CodeExecutionNodeErrorResult,
    CodeExecutionNodeFunctionCallResult,
    CodeExecutionNodeJsonResult,
    CodeExecutionNodeNumberResult,
    CodeExecutionNodeResult,
    CodeExecutionNodeResultData,
    CodeExecutionNodeResultOutput,
    CodeExecutionNodeResultOutput_Array,
    CodeExecutionNodeResultOutput_ChatHistory,
    CodeExecutionNodeResultOutput_Error,
    CodeExecutionNodeResultOutput_FunctionCall,
    CodeExecutionNodeResultOutput_Json,
    CodeExecutionNodeResultOutput_Number,
    CodeExecutionNodeResultOutput_SearchResults,
    CodeExecutionNodeResultOutput_String,
    CodeExecutionNodeSearchResultsResult,
    CodeExecutionNodeStringResult,
    ConditionalNodeResult,
    ConditionalNodeResultData,
    DeploymentProviderPayloadResponse,
    DeploymentRead,
    DeploymentReleaseTagDeploymentHistoryItem,
    DeploymentReleaseTagRead,
    DocumentDocumentToDocumentIndex,
    DocumentIndexChunking,
    DocumentIndexChunkingRequest,
    DocumentIndexChunkingRequest_ReductoChunker,
    DocumentIndexChunkingRequest_SentenceChunker,
    DocumentIndexChunkingRequest_TokenOverlappingWindowChunker,
    DocumentIndexChunking_ReductoChunker,
    DocumentIndexChunking_SentenceChunker,
    DocumentIndexChunking_TokenOverlappingWindowChunker,
    DocumentIndexIndexingConfig,
    DocumentIndexIndexingConfigRequest,
    DocumentIndexRead,
    DocumentRead,
    DocumentStatus,
    EnrichedNormalizedCompletion,
    EntityStatus,
    EnvironmentEnum,
    ErrorEnum,
    ErrorVariableValue,
    ErrorVellumValue,
    ExecutePromptApiErrorResponse,
    ExecutePromptEvent,
    ExecutePromptEvent_Fulfilled,
    ExecutePromptEvent_Initiated,
    ExecutePromptEvent_Rejected,
    ExecutePromptEvent_Streaming,
    ExecutePromptResponse,
    ExecutePromptResponse_Fulfilled,
    ExecutePromptResponse_Rejected,
    ExecuteWorkflowErrorResponse,
    ExecuteWorkflowResponse,
    ExecuteWorkflowStreamErrorResponse,
    ExecuteWorkflowWorkflowResultEvent,
    ExecuteWorkflowWorkflowResultEvent_Fulfilled,
    ExecuteWorkflowWorkflowResultEvent_Rejected,
    ExecutionArrayVellumValue,
    ExecutionChatHistoryVellumValue,
    ExecutionErrorVellumValue,
    ExecutionFunctionCallVellumValue,
    ExecutionJsonVellumValue,
    ExecutionNumberVellumValue,
    ExecutionSearchResultsVellumValue,
    ExecutionStringVellumValue,
    ExecutionVellumValue,
    ExecutionVellumValue_Array,
    ExecutionVellumValue_ChatHistory,
    ExecutionVellumValue_Error,
    ExecutionVellumValue_FunctionCall,
    ExecutionVellumValue_Json,
    ExecutionVellumValue_Number,
    ExecutionVellumValue_SearchResults,
    ExecutionVellumValue_String,
    ExternalTestCaseExecution,
    ExternalTestCaseExecutionRequest,
    FinishReasonEnum,
    FulfilledEnum,
    FulfilledExecutePromptEvent,
    FulfilledExecutePromptResponse,
    FulfilledExecuteWorkflowWorkflowResultEvent,
    FulfilledPromptExecutionMeta,
    FulfilledWorkflowNodeResultEvent,
    FunctionCall,
    FunctionCallChatMessageContent,
    FunctionCallChatMessageContentRequest,
    FunctionCallChatMessageContentValue,
    FunctionCallChatMessageContentValueRequest,
    FunctionCallEnum,
    FunctionCallRequest,
    FunctionCallVariableValue,
    FunctionCallVellumValue,
    GenerateErrorResponse,
    GenerateOptionsRequest,
    GenerateRequest,
    GenerateResponse,
    GenerateResult,
    GenerateResultData,
    GenerateResultError,
    GenerateStreamResponse,
    GenerateStreamResult,
    GenerateStreamResultData,
    HkunlpInstructorXlEnum,
    HkunlpInstructorXlVectorizer,
    HkunlpInstructorXlVectorizerRequest,
    ImageChatMessageContent,
    ImageChatMessageContentRequest,
    ImageEnum,
    ImageVariableValue,
    ImageVellumValue,
    IndexingConfigVectorizer,
    IndexingConfigVectorizerRequest,
    IndexingConfigVectorizerRequest_HkunlpInstructorXl,
    IndexingConfigVectorizerRequest_IntfloatMultilingualE5Large,
    IndexingConfigVectorizerRequest_SentenceTransformersMultiQaMpnetBaseCosV1,
    IndexingConfigVectorizerRequest_SentenceTransformersMultiQaMpnetBaseDotV1,
    IndexingConfigVectorizerRequest_TextEmbedding3Large,
    IndexingConfigVectorizerRequest_TextEmbedding3Small,
    IndexingConfigVectorizerRequest_TextEmbeddingAda002,
    IndexingConfigVectorizer_HkunlpInstructorXl,
    IndexingConfigVectorizer_IntfloatMultilingualE5Large,
    IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseCosV1,
    IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseDotV1,
    IndexingConfigVectorizer_TextEmbedding3Large,
    IndexingConfigVectorizer_TextEmbedding3Small,
    IndexingConfigVectorizer_TextEmbeddingAda002,
    IndexingStateEnum,
    InitiatedEnum,
    InitiatedExecutePromptEvent,
    InitiatedPromptExecutionMeta,
    InitiatedWorkflowNodeResultEvent,
    InstructorVectorizerConfig,
    InstructorVectorizerConfigRequest,
    IntfloatMultilingualE5LargeEnum,
    JsonEnum,
    JsonInputRequest,
    JsonVariableValue,
    JsonVellumValue,
    LogicalOperator,
    LogprobsEnum,
    MergeEnum,
    MergeNodeResult,
    MetadataFilterConfigRequest,
    MetadataFilterRuleCombinator,
    MetadataFilterRuleRequest,
    MetricEnum,
    MetricNodeResult,
    MlModelUsage,
    NamedScenarioInputChatHistoryVariableValueRequest,
    NamedScenarioInputRequest,
    NamedScenarioInputRequest_ChatHistory,
    NamedScenarioInputRequest_String,
    NamedScenarioInputStringVariableValueRequest,
    NamedTestCaseChatHistoryVariableValue,
    NamedTestCaseChatHistoryVariableValueRequest,
    NamedTestCaseErrorVariableValue,
    NamedTestCaseErrorVariableValueRequest,
    NamedTestCaseFunctionCallVariableValue,
    NamedTestCaseFunctionCallVariableValueRequest,
    NamedTestCaseJsonVariableValue,
    NamedTestCaseJsonVariableValueRequest,
    NamedTestCaseNumberVariableValue,
    NamedTestCaseNumberVariableValueRequest,
    NamedTestCaseSearchResultsVariableValue,
    NamedTestCaseSearchResultsVariableValueRequest,
    NamedTestCaseStringVariableValue,
    NamedTestCaseStringVariableValueRequest,
    NamedTestCaseVariableValue,
    NamedTestCaseVariableValueRequest,
    NamedTestCaseVariableValueRequest_ChatHistory,
    NamedTestCaseVariableValueRequest_Error,
    NamedTestCaseVariableValueRequest_FunctionCall,
    NamedTestCaseVariableValueRequest_Json,
    NamedTestCaseVariableValueRequest_Number,
    NamedTestCaseVariableValueRequest_SearchResults,
    NamedTestCaseVariableValueRequest_String,
    NamedTestCaseVariableValue_ChatHistory,
    NamedTestCaseVariableValue_Error,
    NamedTestCaseVariableValue_FunctionCall,
    NamedTestCaseVariableValue_Json,
    NamedTestCaseVariableValue_Number,
    NamedTestCaseVariableValue_SearchResults,
    NamedTestCaseVariableValue_String,
    NodeInputCompiledArrayValue,
    NodeInputCompiledChatHistoryValue,
    NodeInputCompiledErrorValue,
    NodeInputCompiledFunctionCall,
    NodeInputCompiledJsonValue,
    NodeInputCompiledNumberValue,
    NodeInputCompiledSearchResultsValue,
    NodeInputCompiledStringValue,
    NodeInputVariableCompiledValue,
    NodeInputVariableCompiledValue_Array,
    NodeInputVariableCompiledValue_ChatHistory,
    NodeInputVariableCompiledValue_Error,
    NodeInputVariableCompiledValue_FunctionCall,
    NodeInputVariableCompiledValue_Json,
    NodeInputVariableCompiledValue_Number,
    NodeInputVariableCompiledValue_SearchResults,
    NodeInputVariableCompiledValue_String,
    NodeOutputCompiledArrayValue,
    NodeOutputCompiledChatHistoryValue,
    NodeOutputCompiledErrorValue,
    NodeOutputCompiledFunctionCallValue,
    NodeOutputCompiledJsonValue,
    NodeOutputCompiledNumberValue,
    NodeOutputCompiledSearchResultsValue,
    NodeOutputCompiledStringValue,
    NodeOutputCompiledValue,
    NodeOutputCompiledValue_Array,
    NodeOutputCompiledValue_ChatHistory,
    NodeOutputCompiledValue_Error,
    NodeOutputCompiledValue_FunctionCall,
    NodeOutputCompiledValue_Json,
    NodeOutputCompiledValue_Number,
    NodeOutputCompiledValue_SearchResults,
    NodeOutputCompiledValue_String,
    NormalizedLogProbs,
    NormalizedTokenLogProbs,
    NumberEnum,
    NumberVariableValue,
    NumberVellumValue,
    OpenAiVectorizerConfig,
    OpenAiVectorizerConfigRequest,
    OpenAiVectorizerTextEmbedding3Large,
    OpenAiVectorizerTextEmbedding3LargeRequest,
    OpenAiVectorizerTextEmbedding3Small,
    OpenAiVectorizerTextEmbedding3SmallRequest,
    OpenAiVectorizerTextEmbeddingAda002,
    OpenAiVectorizerTextEmbeddingAda002Request,
    PaginatedDocumentIndexReadList,
    PaginatedSlimDeploymentReadList,
    PaginatedSlimDocumentList,
    PaginatedSlimWorkflowDeploymentList,
    PaginatedTestSuiteRunExecutionList,
    PaginatedTestSuiteTestCaseList,
    ProcessingFailureReasonEnum,
    ProcessingStateEnum,
    PromptDeploymentExpandMetaRequestRequest,
    PromptDeploymentInputRequest,
    PromptDeploymentInputRequest_ChatHistory,
    PromptDeploymentInputRequest_Json,
    PromptDeploymentInputRequest_String,
    PromptExecutionMeta,
    PromptNodeResult,
    PromptNodeResultData,
    PromptOutput,
    PromptOutput_Error,
    PromptOutput_FunctionCall,
    PromptOutput_Json,
    PromptOutput_String,
    RawPromptExecutionOverridesRequest,
    ReductoChunkerConfig,
    ReductoChunkerConfigRequest,
    ReductoChunkerEnum,
    ReductoChunking,
    ReductoChunkingRequest,
    RejectedEnum,
    RejectedExecutePromptEvent,
    RejectedExecutePromptResponse,
    RejectedExecuteWorkflowWorkflowResultEvent,
    RejectedPromptExecutionMeta,
    RejectedWorkflowNodeResultEvent,
    ReleaseTagSource,
    SandboxScenario,
    ScenarioInput,
    ScenarioInputChatHistoryVariableValue,
    ScenarioInputStringVariableValue,
    ScenarioInput_ChatHistory,
    ScenarioInput_String,
    SearchErrorResponse,
    SearchFiltersRequest,
    SearchNodeResult,
    SearchNodeResultData,
    SearchRequestOptionsRequest,
    SearchResponse,
    SearchResult,
    SearchResultDocument,
    SearchResultDocumentRequest,
    SearchResultMergingRequest,
    SearchResultRequest,
    SearchResultsEnum,
    SearchWeightsRequest,
    SentenceChunkerConfig,
    SentenceChunkerConfigRequest,
    SentenceChunkerEnum,
    SentenceChunking,
    SentenceChunkingRequest,
    SentenceTransformersMultiQaMpnetBaseCosV1Enum,
    SentenceTransformersMultiQaMpnetBaseDotV1Enum,
    SlimDeploymentRead,
    SlimDocument,
    SlimWorkflowDeployment,
    StreamingEnum,
    StreamingExecutePromptEvent,
    StreamingPromptExecutionMeta,
    StreamingWorkflowNodeResultEvent,
    StringChatMessageContent,
    StringChatMessageContentRequest,
    StringEnum,
    StringInputRequest,
    StringVariableValue,
    StringVellumValue,
    SubmitCompletionActualRequest,
    SubmitCompletionActualsErrorResponse,
    SubmitWorkflowExecutionActualRequest,
    SubmitWorkflowExecutionActualRequest_ChatHistory,
    SubmitWorkflowExecutionActualRequest_Json,
    SubmitWorkflowExecutionActualRequest_String,
    SubworkflowEnum,
    SubworkflowNodeResult,
    SubworkflowNodeResultData,
    TemplatingNodeArrayResult,
    TemplatingNodeChatHistoryResult,
    TemplatingNodeErrorResult,
    TemplatingNodeFunctionCallResult,
    TemplatingNodeJsonResult,
    TemplatingNodeNumberResult,
    TemplatingNodeResult,
    TemplatingNodeResultData,
    TemplatingNodeResultOutput,
    TemplatingNodeResultOutput_Array,
    TemplatingNodeResultOutput_ChatHistory,
    TemplatingNodeResultOutput_Error,
    TemplatingNodeResultOutput_FunctionCall,
    TemplatingNodeResultOutput_Json,
    TemplatingNodeResultOutput_Number,
    TemplatingNodeResultOutput_SearchResults,
    TemplatingNodeResultOutput_String,
    TemplatingNodeSearchResultsResult,
    TemplatingNodeStringResult,
    TerminalNodeArrayResult,
    TerminalNodeChatHistoryResult,
    TerminalNodeErrorResult,
    TerminalNodeFunctionCallResult,
    TerminalNodeJsonResult,
    TerminalNodeNumberResult,
    TerminalNodeResult,
    TerminalNodeResultData,
    TerminalNodeResultOutput,
    TerminalNodeResultOutput_Array,
    TerminalNodeResultOutput_ChatHistory,
    TerminalNodeResultOutput_Error,
    TerminalNodeResultOutput_FunctionCall,
    TerminalNodeResultOutput_Json,
    TerminalNodeResultOutput_Number,
    TerminalNodeResultOutput_SearchResults,
    TerminalNodeResultOutput_String,
    TerminalNodeSearchResultsResult,
    TerminalNodeStringResult,
    TestCaseChatHistoryVariableValue,
    TestCaseErrorVariableValue,
    TestCaseFunctionCallVariableValue,
    TestCaseJsonVariableValue,
    TestCaseNumberVariableValue,
    TestCaseSearchResultsVariableValue,
    TestCaseStringVariableValue,
    TestCaseVariableValue,
    TestCaseVariableValue_ChatHistory,
    TestCaseVariableValue_Error,
    TestCaseVariableValue_FunctionCall,
    TestCaseVariableValue_Json,
    TestCaseVariableValue_Number,
    TestCaseVariableValue_SearchResults,
    TestCaseVariableValue_String,
    TestSuiteRunDeploymentReleaseTagExecConfig,
    TestSuiteRunDeploymentReleaseTagExecConfigData,
    TestSuiteRunDeploymentReleaseTagExecConfigDataRequest,
    TestSuiteRunDeploymentReleaseTagExecConfigRequest,
    TestSuiteRunDeploymentReleaseTagExecConfigTypeEnum,
    TestSuiteRunExecConfig,
    TestSuiteRunExecConfigRequest,
    TestSuiteRunExecConfigRequest_DeploymentReleaseTag,
    TestSuiteRunExecConfigRequest_External,
    TestSuiteRunExecConfigRequest_WorkflowReleaseTag,
    TestSuiteRunExecConfig_DeploymentReleaseTag,
    TestSuiteRunExecConfig_External,
    TestSuiteRunExecConfig_WorkflowReleaseTag,
    TestSuiteRunExecution,
    TestSuiteRunExecutionChatHistoryOutput,
    TestSuiteRunExecutionErrorOutput,
    TestSuiteRunExecutionFunctionCallOutput,
    TestSuiteRunExecutionJsonOutput,
    TestSuiteRunExecutionMetricDefinition,
    TestSuiteRunExecutionMetricResult,
    TestSuiteRunExecutionNumberOutput,
    TestSuiteRunExecutionOutput,
    TestSuiteRunExecutionOutput_ChatHistory,
    TestSuiteRunExecutionOutput_Error,
    TestSuiteRunExecutionOutput_FunctionCall,
    TestSuiteRunExecutionOutput_Json,
    TestSuiteRunExecutionOutput_Number,
    TestSuiteRunExecutionOutput_SearchResults,
    TestSuiteRunExecutionOutput_String,
    TestSuiteRunExecutionSearchResultsOutput,
    TestSuiteRunExecutionStringOutput,
    TestSuiteRunExternalExecConfig,
    TestSuiteRunExternalExecConfigData,
    TestSuiteRunExternalExecConfigDataRequest,
    TestSuiteRunExternalExecConfigRequest,
    TestSuiteRunExternalExecConfigTypeEnum,
    TestSuiteRunMetricErrorOutput,
    TestSuiteRunMetricErrorOutputTypeEnum,
    TestSuiteRunMetricNumberOutput,
    TestSuiteRunMetricNumberOutputTypeEnum,
    TestSuiteRunMetricOutput,
    TestSuiteRunMetricOutput_Error,
    TestSuiteRunMetricOutput_Number,
    TestSuiteRunMetricOutput_String,
    TestSuiteRunMetricStringOutput,
    TestSuiteRunMetricStringOutputTypeEnum,
    TestSuiteRunRead,
    TestSuiteRunState,
    TestSuiteRunTestSuite,
    TestSuiteRunWorkflowReleaseTagExecConfig,
    TestSuiteRunWorkflowReleaseTagExecConfigData,
    TestSuiteRunWorkflowReleaseTagExecConfigDataRequest,
    TestSuiteRunWorkflowReleaseTagExecConfigRequest,
    TestSuiteRunWorkflowReleaseTagExecConfigTypeEnum,
    TestSuiteTestCase,
    TextEmbedding3LargeEnum,
    TextEmbedding3SmallEnum,
    TextEmbeddingAda002Enum,
    TokenOverlappingWindowChunkerConfig,
    TokenOverlappingWindowChunkerConfigRequest,
    TokenOverlappingWindowChunkerEnum,
    TokenOverlappingWindowChunking,
    TokenOverlappingWindowChunkingRequest,
    UploadDocumentErrorResponse,
    UploadDocumentResponse,
    VellumError,
    VellumErrorCodeEnum,
    VellumErrorRequest,
    VellumImage,
    VellumImageRequest,
    VellumVariable,
    VellumVariableType,
    WorkflowDeploymentRead,
    WorkflowEventError,
    WorkflowExecutionActualChatHistoryRequest,
    WorkflowExecutionActualJsonRequest,
    WorkflowExecutionActualStringRequest,
    WorkflowExecutionEventErrorCode,
    WorkflowExecutionEventType,
    WorkflowExecutionNodeResultEvent,
    WorkflowExecutionWorkflowResultEvent,
    WorkflowNodeResultData,
    WorkflowNodeResultData_Api,
    WorkflowNodeResultData_CodeExecution,
    WorkflowNodeResultData_Conditional,
    WorkflowNodeResultData_Merge,
    WorkflowNodeResultData_Metric,
    WorkflowNodeResultData_Prompt,
    WorkflowNodeResultData_Search,
    WorkflowNodeResultData_Subworkflow,
    WorkflowNodeResultData_Templating,
    WorkflowNodeResultData_Terminal,
    WorkflowNodeResultEvent,
    WorkflowNodeResultEventState,
    WorkflowNodeResultEvent_Fulfilled,
    WorkflowNodeResultEvent_Initiated,
    WorkflowNodeResultEvent_Rejected,
    WorkflowNodeResultEvent_Streaming,
    WorkflowOutput,
    WorkflowOutputArray,
    WorkflowOutputChatHistory,
    WorkflowOutputError,
    WorkflowOutputFunctionCall,
    WorkflowOutputImage,
    WorkflowOutputJson,
    WorkflowOutputNumber,
    WorkflowOutputSearchResults,
    WorkflowOutputString,
    WorkflowOutput_Array,
    WorkflowOutput_ChatHistory,
    WorkflowOutput_Error,
    WorkflowOutput_FunctionCall,
    WorkflowOutput_Image,
    WorkflowOutput_Json,
    WorkflowOutput_Number,
    WorkflowOutput_SearchResults,
    WorkflowOutput_String,
    WorkflowReleaseTagRead,
    WorkflowReleaseTagWorkflowDeploymentHistoryItem,
    WorkflowRequestChatHistoryInputRequest,
    WorkflowRequestInputRequest,
    WorkflowRequestInputRequest_ChatHistory,
    WorkflowRequestInputRequest_Json,
    WorkflowRequestInputRequest_Number,
    WorkflowRequestInputRequest_String,
    WorkflowRequestJsonInputRequest,
    WorkflowRequestNumberInputRequest,
    WorkflowRequestStringInputRequest,
    WorkflowResultEvent,
    WorkflowResultEventOutputData,
    WorkflowResultEventOutputDataArray,
    WorkflowResultEventOutputDataChatHistory,
    WorkflowResultEventOutputDataError,
    WorkflowResultEventOutputDataFunctionCall,
    WorkflowResultEventOutputDataJson,
    WorkflowResultEventOutputDataNumber,
    WorkflowResultEventOutputDataSearchResults,
    WorkflowResultEventOutputDataString,
    WorkflowResultEventOutputData_Array,
    WorkflowResultEventOutputData_ChatHistory,
    WorkflowResultEventOutputData_Error,
    WorkflowResultEventOutputData_FunctionCall,
    WorkflowResultEventOutputData_Json,
    WorkflowResultEventOutputData_Number,
    WorkflowResultEventOutputData_SearchResults,
    WorkflowResultEventOutputData_String,
    WorkflowStreamEvent,
    WorkflowStreamEvent_Node,
    WorkflowStreamEvent_Workflow,
)
from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError
from .resources import (
    DeploymentsListRequestStatus,
    DocumentIndexesListRequestStatus,
    WorkflowDeploymentsListRequestStatus,
    deployments,
    document_indexes,
    documents,
    folder_entities,
    sandboxes,
    test_suite_runs,
    test_suites,
    workflow_deployments,
    workflow_sandboxes,
)
from .environment import VellumEnvironment
from .version import __version__

__all__ = [
    "AddOpenaiApiKeyEnum",
    "ApiNodeResult",
    "ApiNodeResultData",
    "ArrayChatMessageContent",
    "ArrayChatMessageContentItem",
    "ArrayChatMessageContentItemRequest",
    "ArrayChatMessageContentItemRequest_FunctionCall",
    "ArrayChatMessageContentItemRequest_Image",
    "ArrayChatMessageContentItemRequest_String",
    "ArrayChatMessageContentItem_FunctionCall",
    "ArrayChatMessageContentItem_Image",
    "ArrayChatMessageContentItem_String",
    "ArrayChatMessageContentRequest",
    "ArrayEnum",
    "ArrayVariableValueItem",
    "ArrayVariableValueItem_Error",
    "ArrayVariableValueItem_FunctionCall",
    "ArrayVariableValueItem_Image",
    "ArrayVariableValueItem_Json",
    "ArrayVariableValueItem_Number",
    "ArrayVariableValueItem_String",
    "ArrayVellumValueItem",
    "ArrayVellumValueItem_Error",
    "ArrayVellumValueItem_FunctionCall",
    "ArrayVellumValueItem_Image",
    "ArrayVellumValueItem_Json",
    "ArrayVellumValueItem_Number",
    "ArrayVellumValueItem_String",
    "BadRequestError",
    "BasicVectorizerIntfloatMultilingualE5Large",
    "BasicVectorizerIntfloatMultilingualE5LargeRequest",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request",
    "ChatHistoryEnum",
    "ChatHistoryInputRequest",
    "ChatMessage",
    "ChatMessageContent",
    "ChatMessageContentRequest",
    "ChatMessageContentRequest_Array",
    "ChatMessageContentRequest_FunctionCall",
    "ChatMessageContentRequest_Image",
    "ChatMessageContentRequest_String",
    "ChatMessageContent_Array",
    "ChatMessageContent_FunctionCall",
    "ChatMessageContent_Image",
    "ChatMessageContent_String",
    "ChatMessageRequest",
    "ChatMessageRole",
    "CodeExecutionNodeArrayResult",
    "CodeExecutionNodeChatHistoryResult",
    "CodeExecutionNodeErrorResult",
    "CodeExecutionNodeFunctionCallResult",
    "CodeExecutionNodeJsonResult",
    "CodeExecutionNodeNumberResult",
    "CodeExecutionNodeResult",
    "CodeExecutionNodeResultData",
    "CodeExecutionNodeResultOutput",
    "CodeExecutionNodeResultOutput_Array",
    "CodeExecutionNodeResultOutput_ChatHistory",
    "CodeExecutionNodeResultOutput_Error",
    "CodeExecutionNodeResultOutput_FunctionCall",
    "CodeExecutionNodeResultOutput_Json",
    "CodeExecutionNodeResultOutput_Number",
    "CodeExecutionNodeResultOutput_SearchResults",
    "CodeExecutionNodeResultOutput_String",
    "CodeExecutionNodeSearchResultsResult",
    "CodeExecutionNodeStringResult",
    "ConditionalNodeResult",
    "ConditionalNodeResultData",
    "DeploymentProviderPayloadResponse",
    "DeploymentRead",
    "DeploymentReleaseTagDeploymentHistoryItem",
    "DeploymentReleaseTagRead",
    "DeploymentsListRequestStatus",
    "DocumentDocumentToDocumentIndex",
    "DocumentIndexChunking",
    "DocumentIndexChunkingRequest",
    "DocumentIndexChunkingRequest_ReductoChunker",
    "DocumentIndexChunkingRequest_SentenceChunker",
    "DocumentIndexChunkingRequest_TokenOverlappingWindowChunker",
    "DocumentIndexChunking_ReductoChunker",
    "DocumentIndexChunking_SentenceChunker",
    "DocumentIndexChunking_TokenOverlappingWindowChunker",
    "DocumentIndexIndexingConfig",
    "DocumentIndexIndexingConfigRequest",
    "DocumentIndexRead",
    "DocumentIndexesListRequestStatus",
    "DocumentRead",
    "DocumentStatus",
    "EnrichedNormalizedCompletion",
    "EntityStatus",
    "EnvironmentEnum",
    "ErrorEnum",
    "ErrorVariableValue",
    "ErrorVellumValue",
    "ExecutePromptApiErrorResponse",
    "ExecutePromptEvent",
    "ExecutePromptEvent_Fulfilled",
    "ExecutePromptEvent_Initiated",
    "ExecutePromptEvent_Rejected",
    "ExecutePromptEvent_Streaming",
    "ExecutePromptResponse",
    "ExecutePromptResponse_Fulfilled",
    "ExecutePromptResponse_Rejected",
    "ExecuteWorkflowErrorResponse",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowStreamErrorResponse",
    "ExecuteWorkflowWorkflowResultEvent",
    "ExecuteWorkflowWorkflowResultEvent_Fulfilled",
    "ExecuteWorkflowWorkflowResultEvent_Rejected",
    "ExecutionArrayVellumValue",
    "ExecutionChatHistoryVellumValue",
    "ExecutionErrorVellumValue",
    "ExecutionFunctionCallVellumValue",
    "ExecutionJsonVellumValue",
    "ExecutionNumberVellumValue",
    "ExecutionSearchResultsVellumValue",
    "ExecutionStringVellumValue",
    "ExecutionVellumValue",
    "ExecutionVellumValue_Array",
    "ExecutionVellumValue_ChatHistory",
    "ExecutionVellumValue_Error",
    "ExecutionVellumValue_FunctionCall",
    "ExecutionVellumValue_Json",
    "ExecutionVellumValue_Number",
    "ExecutionVellumValue_SearchResults",
    "ExecutionVellumValue_String",
    "ExternalTestCaseExecution",
    "ExternalTestCaseExecutionRequest",
    "FinishReasonEnum",
    "ForbiddenError",
    "FulfilledEnum",
    "FulfilledExecutePromptEvent",
    "FulfilledExecutePromptResponse",
    "FulfilledExecuteWorkflowWorkflowResultEvent",
    "FulfilledPromptExecutionMeta",
    "FulfilledWorkflowNodeResultEvent",
    "FunctionCall",
    "FunctionCallChatMessageContent",
    "FunctionCallChatMessageContentRequest",
    "FunctionCallChatMessageContentValue",
    "FunctionCallChatMessageContentValueRequest",
    "FunctionCallEnum",
    "FunctionCallRequest",
    "FunctionCallVariableValue",
    "FunctionCallVellumValue",
    "GenerateErrorResponse",
    "GenerateOptionsRequest",
    "GenerateRequest",
    "GenerateResponse",
    "GenerateResult",
    "GenerateResultData",
    "GenerateResultError",
    "GenerateStreamResponse",
    "GenerateStreamResult",
    "GenerateStreamResultData",
    "HkunlpInstructorXlEnum",
    "HkunlpInstructorXlVectorizer",
    "HkunlpInstructorXlVectorizerRequest",
    "ImageChatMessageContent",
    "ImageChatMessageContentRequest",
    "ImageEnum",
    "ImageVariableValue",
    "ImageVellumValue",
    "IndexingConfigVectorizer",
    "IndexingConfigVectorizerRequest",
    "IndexingConfigVectorizerRequest_HkunlpInstructorXl",
    "IndexingConfigVectorizerRequest_IntfloatMultilingualE5Large",
    "IndexingConfigVectorizerRequest_SentenceTransformersMultiQaMpnetBaseCosV1",
    "IndexingConfigVectorizerRequest_SentenceTransformersMultiQaMpnetBaseDotV1",
    "IndexingConfigVectorizerRequest_TextEmbedding3Large",
    "IndexingConfigVectorizerRequest_TextEmbedding3Small",
    "IndexingConfigVectorizerRequest_TextEmbeddingAda002",
    "IndexingConfigVectorizer_HkunlpInstructorXl",
    "IndexingConfigVectorizer_IntfloatMultilingualE5Large",
    "IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseCosV1",
    "IndexingConfigVectorizer_SentenceTransformersMultiQaMpnetBaseDotV1",
    "IndexingConfigVectorizer_TextEmbedding3Large",
    "IndexingConfigVectorizer_TextEmbedding3Small",
    "IndexingConfigVectorizer_TextEmbeddingAda002",
    "IndexingStateEnum",
    "InitiatedEnum",
    "InitiatedExecutePromptEvent",
    "InitiatedPromptExecutionMeta",
    "InitiatedWorkflowNodeResultEvent",
    "InstructorVectorizerConfig",
    "InstructorVectorizerConfigRequest",
    "InternalServerError",
    "IntfloatMultilingualE5LargeEnum",
    "JsonEnum",
    "JsonInputRequest",
    "JsonVariableValue",
    "JsonVellumValue",
    "LogicalOperator",
    "LogprobsEnum",
    "MergeEnum",
    "MergeNodeResult",
    "MetadataFilterConfigRequest",
    "MetadataFilterRuleCombinator",
    "MetadataFilterRuleRequest",
    "MetricEnum",
    "MetricNodeResult",
    "MlModelUsage",
    "NamedScenarioInputChatHistoryVariableValueRequest",
    "NamedScenarioInputRequest",
    "NamedScenarioInputRequest_ChatHistory",
    "NamedScenarioInputRequest_String",
    "NamedScenarioInputStringVariableValueRequest",
    "NamedTestCaseChatHistoryVariableValue",
    "NamedTestCaseChatHistoryVariableValueRequest",
    "NamedTestCaseErrorVariableValue",
    "NamedTestCaseErrorVariableValueRequest",
    "NamedTestCaseFunctionCallVariableValue",
    "NamedTestCaseFunctionCallVariableValueRequest",
    "NamedTestCaseJsonVariableValue",
    "NamedTestCaseJsonVariableValueRequest",
    "NamedTestCaseNumberVariableValue",
    "NamedTestCaseNumberVariableValueRequest",
    "NamedTestCaseSearchResultsVariableValue",
    "NamedTestCaseSearchResultsVariableValueRequest",
    "NamedTestCaseStringVariableValue",
    "NamedTestCaseStringVariableValueRequest",
    "NamedTestCaseVariableValue",
    "NamedTestCaseVariableValueRequest",
    "NamedTestCaseVariableValueRequest_ChatHistory",
    "NamedTestCaseVariableValueRequest_Error",
    "NamedTestCaseVariableValueRequest_FunctionCall",
    "NamedTestCaseVariableValueRequest_Json",
    "NamedTestCaseVariableValueRequest_Number",
    "NamedTestCaseVariableValueRequest_SearchResults",
    "NamedTestCaseVariableValueRequest_String",
    "NamedTestCaseVariableValue_ChatHistory",
    "NamedTestCaseVariableValue_Error",
    "NamedTestCaseVariableValue_FunctionCall",
    "NamedTestCaseVariableValue_Json",
    "NamedTestCaseVariableValue_Number",
    "NamedTestCaseVariableValue_SearchResults",
    "NamedTestCaseVariableValue_String",
    "NodeInputCompiledArrayValue",
    "NodeInputCompiledChatHistoryValue",
    "NodeInputCompiledErrorValue",
    "NodeInputCompiledFunctionCall",
    "NodeInputCompiledJsonValue",
    "NodeInputCompiledNumberValue",
    "NodeInputCompiledSearchResultsValue",
    "NodeInputCompiledStringValue",
    "NodeInputVariableCompiledValue",
    "NodeInputVariableCompiledValue_Array",
    "NodeInputVariableCompiledValue_ChatHistory",
    "NodeInputVariableCompiledValue_Error",
    "NodeInputVariableCompiledValue_FunctionCall",
    "NodeInputVariableCompiledValue_Json",
    "NodeInputVariableCompiledValue_Number",
    "NodeInputVariableCompiledValue_SearchResults",
    "NodeInputVariableCompiledValue_String",
    "NodeOutputCompiledArrayValue",
    "NodeOutputCompiledChatHistoryValue",
    "NodeOutputCompiledErrorValue",
    "NodeOutputCompiledFunctionCallValue",
    "NodeOutputCompiledJsonValue",
    "NodeOutputCompiledNumberValue",
    "NodeOutputCompiledSearchResultsValue",
    "NodeOutputCompiledStringValue",
    "NodeOutputCompiledValue",
    "NodeOutputCompiledValue_Array",
    "NodeOutputCompiledValue_ChatHistory",
    "NodeOutputCompiledValue_Error",
    "NodeOutputCompiledValue_FunctionCall",
    "NodeOutputCompiledValue_Json",
    "NodeOutputCompiledValue_Number",
    "NodeOutputCompiledValue_SearchResults",
    "NodeOutputCompiledValue_String",
    "NormalizedLogProbs",
    "NormalizedTokenLogProbs",
    "NotFoundError",
    "NumberEnum",
    "NumberVariableValue",
    "NumberVellumValue",
    "OpenAiVectorizerConfig",
    "OpenAiVectorizerConfigRequest",
    "OpenAiVectorizerTextEmbedding3Large",
    "OpenAiVectorizerTextEmbedding3LargeRequest",
    "OpenAiVectorizerTextEmbedding3Small",
    "OpenAiVectorizerTextEmbedding3SmallRequest",
    "OpenAiVectorizerTextEmbeddingAda002",
    "OpenAiVectorizerTextEmbeddingAda002Request",
    "PaginatedDocumentIndexReadList",
    "PaginatedSlimDeploymentReadList",
    "PaginatedSlimDocumentList",
    "PaginatedSlimWorkflowDeploymentList",
    "PaginatedTestSuiteRunExecutionList",
    "PaginatedTestSuiteTestCaseList",
    "ProcessingFailureReasonEnum",
    "ProcessingStateEnum",
    "PromptDeploymentExpandMetaRequestRequest",
    "PromptDeploymentInputRequest",
    "PromptDeploymentInputRequest_ChatHistory",
    "PromptDeploymentInputRequest_Json",
    "PromptDeploymentInputRequest_String",
    "PromptExecutionMeta",
    "PromptNodeResult",
    "PromptNodeResultData",
    "PromptOutput",
    "PromptOutput_Error",
    "PromptOutput_FunctionCall",
    "PromptOutput_Json",
    "PromptOutput_String",
    "RawPromptExecutionOverridesRequest",
    "ReductoChunkerConfig",
    "ReductoChunkerConfigRequest",
    "ReductoChunkerEnum",
    "ReductoChunking",
    "ReductoChunkingRequest",
    "RejectedEnum",
    "RejectedExecutePromptEvent",
    "RejectedExecutePromptResponse",
    "RejectedExecuteWorkflowWorkflowResultEvent",
    "RejectedPromptExecutionMeta",
    "RejectedWorkflowNodeResultEvent",
    "ReleaseTagSource",
    "SandboxScenario",
    "ScenarioInput",
    "ScenarioInputChatHistoryVariableValue",
    "ScenarioInputStringVariableValue",
    "ScenarioInput_ChatHistory",
    "ScenarioInput_String",
    "SearchErrorResponse",
    "SearchFiltersRequest",
    "SearchNodeResult",
    "SearchNodeResultData",
    "SearchRequestOptionsRequest",
    "SearchResponse",
    "SearchResult",
    "SearchResultDocument",
    "SearchResultDocumentRequest",
    "SearchResultMergingRequest",
    "SearchResultRequest",
    "SearchResultsEnum",
    "SearchWeightsRequest",
    "SentenceChunkerConfig",
    "SentenceChunkerConfigRequest",
    "SentenceChunkerEnum",
    "SentenceChunking",
    "SentenceChunkingRequest",
    "SentenceTransformersMultiQaMpnetBaseCosV1Enum",
    "SentenceTransformersMultiQaMpnetBaseDotV1Enum",
    "SlimDeploymentRead",
    "SlimDocument",
    "SlimWorkflowDeployment",
    "StreamingEnum",
    "StreamingExecutePromptEvent",
    "StreamingPromptExecutionMeta",
    "StreamingWorkflowNodeResultEvent",
    "StringChatMessageContent",
    "StringChatMessageContentRequest",
    "StringEnum",
    "StringInputRequest",
    "StringVariableValue",
    "StringVellumValue",
    "SubmitCompletionActualRequest",
    "SubmitCompletionActualsErrorResponse",
    "SubmitWorkflowExecutionActualRequest",
    "SubmitWorkflowExecutionActualRequest_ChatHistory",
    "SubmitWorkflowExecutionActualRequest_Json",
    "SubmitWorkflowExecutionActualRequest_String",
    "SubworkflowEnum",
    "SubworkflowNodeResult",
    "SubworkflowNodeResultData",
    "TemplatingNodeArrayResult",
    "TemplatingNodeChatHistoryResult",
    "TemplatingNodeErrorResult",
    "TemplatingNodeFunctionCallResult",
    "TemplatingNodeJsonResult",
    "TemplatingNodeNumberResult",
    "TemplatingNodeResult",
    "TemplatingNodeResultData",
    "TemplatingNodeResultOutput",
    "TemplatingNodeResultOutput_Array",
    "TemplatingNodeResultOutput_ChatHistory",
    "TemplatingNodeResultOutput_Error",
    "TemplatingNodeResultOutput_FunctionCall",
    "TemplatingNodeResultOutput_Json",
    "TemplatingNodeResultOutput_Number",
    "TemplatingNodeResultOutput_SearchResults",
    "TemplatingNodeResultOutput_String",
    "TemplatingNodeSearchResultsResult",
    "TemplatingNodeStringResult",
    "TerminalNodeArrayResult",
    "TerminalNodeChatHistoryResult",
    "TerminalNodeErrorResult",
    "TerminalNodeFunctionCallResult",
    "TerminalNodeJsonResult",
    "TerminalNodeNumberResult",
    "TerminalNodeResult",
    "TerminalNodeResultData",
    "TerminalNodeResultOutput",
    "TerminalNodeResultOutput_Array",
    "TerminalNodeResultOutput_ChatHistory",
    "TerminalNodeResultOutput_Error",
    "TerminalNodeResultOutput_FunctionCall",
    "TerminalNodeResultOutput_Json",
    "TerminalNodeResultOutput_Number",
    "TerminalNodeResultOutput_SearchResults",
    "TerminalNodeResultOutput_String",
    "TerminalNodeSearchResultsResult",
    "TerminalNodeStringResult",
    "TestCaseChatHistoryVariableValue",
    "TestCaseErrorVariableValue",
    "TestCaseFunctionCallVariableValue",
    "TestCaseJsonVariableValue",
    "TestCaseNumberVariableValue",
    "TestCaseSearchResultsVariableValue",
    "TestCaseStringVariableValue",
    "TestCaseVariableValue",
    "TestCaseVariableValue_ChatHistory",
    "TestCaseVariableValue_Error",
    "TestCaseVariableValue_FunctionCall",
    "TestCaseVariableValue_Json",
    "TestCaseVariableValue_Number",
    "TestCaseVariableValue_SearchResults",
    "TestCaseVariableValue_String",
    "TestSuiteRunDeploymentReleaseTagExecConfig",
    "TestSuiteRunDeploymentReleaseTagExecConfigData",
    "TestSuiteRunDeploymentReleaseTagExecConfigDataRequest",
    "TestSuiteRunDeploymentReleaseTagExecConfigRequest",
    "TestSuiteRunDeploymentReleaseTagExecConfigTypeEnum",
    "TestSuiteRunExecConfig",
    "TestSuiteRunExecConfigRequest",
    "TestSuiteRunExecConfigRequest_DeploymentReleaseTag",
    "TestSuiteRunExecConfigRequest_External",
    "TestSuiteRunExecConfigRequest_WorkflowReleaseTag",
    "TestSuiteRunExecConfig_DeploymentReleaseTag",
    "TestSuiteRunExecConfig_External",
    "TestSuiteRunExecConfig_WorkflowReleaseTag",
    "TestSuiteRunExecution",
    "TestSuiteRunExecutionChatHistoryOutput",
    "TestSuiteRunExecutionErrorOutput",
    "TestSuiteRunExecutionFunctionCallOutput",
    "TestSuiteRunExecutionJsonOutput",
    "TestSuiteRunExecutionMetricDefinition",
    "TestSuiteRunExecutionMetricResult",
    "TestSuiteRunExecutionNumberOutput",
    "TestSuiteRunExecutionOutput",
    "TestSuiteRunExecutionOutput_ChatHistory",
    "TestSuiteRunExecutionOutput_Error",
    "TestSuiteRunExecutionOutput_FunctionCall",
    "TestSuiteRunExecutionOutput_Json",
    "TestSuiteRunExecutionOutput_Number",
    "TestSuiteRunExecutionOutput_SearchResults",
    "TestSuiteRunExecutionOutput_String",
    "TestSuiteRunExecutionSearchResultsOutput",
    "TestSuiteRunExecutionStringOutput",
    "TestSuiteRunExternalExecConfig",
    "TestSuiteRunExternalExecConfigData",
    "TestSuiteRunExternalExecConfigDataRequest",
    "TestSuiteRunExternalExecConfigRequest",
    "TestSuiteRunExternalExecConfigTypeEnum",
    "TestSuiteRunMetricErrorOutput",
    "TestSuiteRunMetricErrorOutputTypeEnum",
    "TestSuiteRunMetricNumberOutput",
    "TestSuiteRunMetricNumberOutputTypeEnum",
    "TestSuiteRunMetricOutput",
    "TestSuiteRunMetricOutput_Error",
    "TestSuiteRunMetricOutput_Number",
    "TestSuiteRunMetricOutput_String",
    "TestSuiteRunMetricStringOutput",
    "TestSuiteRunMetricStringOutputTypeEnum",
    "TestSuiteRunRead",
    "TestSuiteRunState",
    "TestSuiteRunTestSuite",
    "TestSuiteRunWorkflowReleaseTagExecConfig",
    "TestSuiteRunWorkflowReleaseTagExecConfigData",
    "TestSuiteRunWorkflowReleaseTagExecConfigDataRequest",
    "TestSuiteRunWorkflowReleaseTagExecConfigRequest",
    "TestSuiteRunWorkflowReleaseTagExecConfigTypeEnum",
    "TestSuiteTestCase",
    "TextEmbedding3LargeEnum",
    "TextEmbedding3SmallEnum",
    "TextEmbeddingAda002Enum",
    "TokenOverlappingWindowChunkerConfig",
    "TokenOverlappingWindowChunkerConfigRequest",
    "TokenOverlappingWindowChunkerEnum",
    "TokenOverlappingWindowChunking",
    "TokenOverlappingWindowChunkingRequest",
    "UploadDocumentErrorResponse",
    "UploadDocumentResponse",
    "VellumEnvironment",
    "VellumError",
    "VellumErrorCodeEnum",
    "VellumErrorRequest",
    "VellumImage",
    "VellumImageRequest",
    "VellumVariable",
    "VellumVariableType",
    "WorkflowDeploymentRead",
    "WorkflowDeploymentsListRequestStatus",
    "WorkflowEventError",
    "WorkflowExecutionActualChatHistoryRequest",
    "WorkflowExecutionActualJsonRequest",
    "WorkflowExecutionActualStringRequest",
    "WorkflowExecutionEventErrorCode",
    "WorkflowExecutionEventType",
    "WorkflowExecutionNodeResultEvent",
    "WorkflowExecutionWorkflowResultEvent",
    "WorkflowNodeResultData",
    "WorkflowNodeResultData_Api",
    "WorkflowNodeResultData_CodeExecution",
    "WorkflowNodeResultData_Conditional",
    "WorkflowNodeResultData_Merge",
    "WorkflowNodeResultData_Metric",
    "WorkflowNodeResultData_Prompt",
    "WorkflowNodeResultData_Search",
    "WorkflowNodeResultData_Subworkflow",
    "WorkflowNodeResultData_Templating",
    "WorkflowNodeResultData_Terminal",
    "WorkflowNodeResultEvent",
    "WorkflowNodeResultEventState",
    "WorkflowNodeResultEvent_Fulfilled",
    "WorkflowNodeResultEvent_Initiated",
    "WorkflowNodeResultEvent_Rejected",
    "WorkflowNodeResultEvent_Streaming",
    "WorkflowOutput",
    "WorkflowOutputArray",
    "WorkflowOutputChatHistory",
    "WorkflowOutputError",
    "WorkflowOutputFunctionCall",
    "WorkflowOutputImage",
    "WorkflowOutputJson",
    "WorkflowOutputNumber",
    "WorkflowOutputSearchResults",
    "WorkflowOutputString",
    "WorkflowOutput_Array",
    "WorkflowOutput_ChatHistory",
    "WorkflowOutput_Error",
    "WorkflowOutput_FunctionCall",
    "WorkflowOutput_Image",
    "WorkflowOutput_Json",
    "WorkflowOutput_Number",
    "WorkflowOutput_SearchResults",
    "WorkflowOutput_String",
    "WorkflowReleaseTagRead",
    "WorkflowReleaseTagWorkflowDeploymentHistoryItem",
    "WorkflowRequestChatHistoryInputRequest",
    "WorkflowRequestInputRequest",
    "WorkflowRequestInputRequest_ChatHistory",
    "WorkflowRequestInputRequest_Json",
    "WorkflowRequestInputRequest_Number",
    "WorkflowRequestInputRequest_String",
    "WorkflowRequestJsonInputRequest",
    "WorkflowRequestNumberInputRequest",
    "WorkflowRequestStringInputRequest",
    "WorkflowResultEvent",
    "WorkflowResultEventOutputData",
    "WorkflowResultEventOutputDataArray",
    "WorkflowResultEventOutputDataChatHistory",
    "WorkflowResultEventOutputDataError",
    "WorkflowResultEventOutputDataFunctionCall",
    "WorkflowResultEventOutputDataJson",
    "WorkflowResultEventOutputDataNumber",
    "WorkflowResultEventOutputDataSearchResults",
    "WorkflowResultEventOutputDataString",
    "WorkflowResultEventOutputData_Array",
    "WorkflowResultEventOutputData_ChatHistory",
    "WorkflowResultEventOutputData_Error",
    "WorkflowResultEventOutputData_FunctionCall",
    "WorkflowResultEventOutputData_Json",
    "WorkflowResultEventOutputData_Number",
    "WorkflowResultEventOutputData_SearchResults",
    "WorkflowResultEventOutputData_String",
    "WorkflowStreamEvent",
    "WorkflowStreamEvent_Node",
    "WorkflowStreamEvent_Workflow",
    "__version__",
    "deployments",
    "document_indexes",
    "documents",
    "folder_entities",
    "sandboxes",
    "test_suite_runs",
    "test_suites",
    "workflow_deployments",
    "workflow_sandboxes",
]
